/**
 * The Stats class contains all the stats of the kobold. This includes:
 *      -Agility
 *      -Charisma
 *      -Endurance
 *      -Intelligence
 *      -Luck
 *      -Strength
 */
function Stats() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var max = 10;
    var min = 1;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    
    /**
     * Fetches and returns the name of the member of the passed stat.
     * @param {StatsEnum} stat The stat of which to get the member name.
     * @returns {string} The name of the public member.
     */
    function getMember(stat){
        return StatsEnum.properties[stat].member;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.agility = 5;
    this.charisma = 5;
    this.endurance = 5;
    this.intelligence = 5;
    this.luck = 5;
    this.strength = 5;
    
    this.statMin = min;
    this.statMax = max;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Gets the value of the passed stat.
     * @param {StatsEnum} stat The stat to get.
     * @returns {int} The value of the stat.
     */
    this.get = function(stat) {
        return that[getMember(stat)];
    };
    
    /**
     * Sets the value of the passed stat.
     * @param {StatsEnum} stat The stat to set.
     * @param {int} value The new value to set the stat to.
     */
    this.set = function(stat, value) {
        that[getMember(stat)] = value;
    };
    
    /**
     * Increases the value of the passed stat, to a maximum value.
     * @param {StatsEnum} stat The stat to increase.
     * @param {int} value How much to increase it.
     */
    this.increaseStat = function(stat, value) {
        var curr = that.get(stat);
        var increased = curr + value;
        if(increased > max) {
            increased = max;
        }
        that.set(stat, increased);
    };
    
    /**
     * Decreases the value of the passed stat, to a minimum value.
     * @param {StatsEnum} stat The stat to decrease.
     * @param {int} value How much to decrease it.
     */
    this.decreaseStat = function(stat, value) {
        var curr = that.get(stat);
        var decreased = curr - value;
        if(decreased < min) {
            decreased = min;
        }
        that.set(stat, decreased);
    };
    
    /**
     * Gets the name of the passed stat.
     * @param {StatsEnum} stat The stat to get the name of.
     * @returns {int} The name of the stat.
     */
    this.getName = function(stat) {
        return StatsEnum.properties[stat].name;
    };
    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Stats-stripped} stats A Stats object, stripped of private members and all functions.
     */
    this.load = function (stats) {
        that.agility = stats.agility;
        that.charisma = stats.charisma;
        that.endurance = stats.endurance;
        that.intelligence = stats.intelligence;
        that.luck = stats.luck;
        that.strength = stats.strength;
    };
}
