/**
 * Information about the class.
 */
function LockPickingTumbler(id, key, master, position, active) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var upTop = "90px";
    var downTop = "250px";

    var uniqueIdentifier = arguments.callee.name;
    var elementId = id + "_" + key + "_" + uniqueIdentifier + position;

    var ctrl = new ChoiceController(id, elementId);

    if (!active) {
        ctrl.set(true);
    }

    var image = new LockPickingPreloadingTumbler();

    var upStart;
    var upTime;
    var upGrace;

    var downTime = 750;
    var fastDownTime = 300;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Animates the tumbler going up.
     */
    function animateUp() {
        that.inAnimation = true;
        $("#" + elementId).animate({top: upTop}, upTime, animateDownSlow);
    }

    /**
     * Animates the tumbler going down slowly.
     */
    function animateDownSlow() {
        $("#" + elementId).animate({top: downTop}, downTime, animateDone);
    }

    /**
     * Called when the animation is over.
     */
    function animateDone() {
        that.inAnimation = false;
    }

    /**
     * Tries to set the tumbler in place.
     */
    function inPlace() {
        if (!that.inAnimation) {
            return;
        }
        if(clickCheck()) {
            ctrl.set(true);
            $("#" + elementId).stop(true, false);
            that.inAnimation = false;
            master.done(position);
        } else {
            master.fail(position);
        }
    }

    /**
     * Checks whether the tumbler was successfully set in place.
     * @returns {boolean} True if the setting was successful, false otherwise.
     */
    function clickCheck() {
        var timeElapsed = +new Date() - upStart;
        console.log(upTime, Math.abs(upTime - timeElapsed));
        var absoluteDifference = Math.abs(upTime - timeElapsed);
        return absoluteDifference < Math.ceil(upGrace / 2);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.inAnimation = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    this.build = function () {
        var top = ctrl.get() ? upTop : downTop;
        var tumbler = `<img class="tumbler tumbler${position}" src=${image.url} id="${elementId}" style="top: ${top};">`;
        //var clickbox = `<div class="tumblerclickbox tumblerclickbox${position}"></div>`;
        var clickbox = "";
        return tumbler + clickbox;
    };

    this.registerListeners = function () {
        $("#" + elementId).click(inPlace);
    };

    /**
     * Causes this tumbler to go up.
     * @param {int} time How quickly it should go up, in milliseconds.
     * @param {int} grace Distributed around the peak of the tumbler's trajectory, how long in milliseconds the user has to click it to succeed.
     */
    this.up = function (time, grace) {
        if (ctrl.get()) {
            return;
        }
        upStart = +new Date();
        upTime = time;
        upGrace = grace;
        animateUp();
    };
    
    /**
     * Causes this tumbler to go down.
     */
    this.reset = function() {
        if (!active) {
            return;
        }

        ctrl.set(false);
        $("#" + elementId).stop(true, false);
        that.inAnimation = true;
        $("#" + elementId).animate({top: downTop}, fastDownTime, animateDone);
    };
}

