/**
 * Listens to changes in the localstorage settings.
 * Converts the localstorage from an object to JSON, and then passes it to all registered sublisteners.
 * @param {string} settingsKey The storage-key for settings.
 */
function SettingMasterListener(settingsKey) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // Array of listener functions.
    var listeners = [];

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    /**
     * Registers a listener.
     * It will be passed both the old value and the new value of the settings-object (in that order).
     * @param {function} fun The function to unregister.
     */
    this.register = function (fun) {
        listeners.push(fun);
    };

    /**
     * Unregisters a listener.
     * @param {function} fun The function to unregister.
     */
    this.unregister = function (fun) {
        var index = listeners.indexOf(fun);
        if (index < 0 || index >= listeners.length) {
            return;
        }
        listeners.splice(index, 1);
    };

    /**
     * Add the storage-event listener to the window.
     * Will listen only to changes in settings.
     */
    window.addEventListener('storage', function (event) {
        if (event.key !== settingsKey) {
            return;
        }

        var oldValue = JSON.parse(event.oldValue);
        var newValue = JSON.parse(event.newValue);

        for (var i = 0; i < listeners.length; i++) {
            listeners[i](oldValue, newValue);
        }
    });
}

