/**
 * Controller for the Caravan Confinement scene.
 * @param {string} sceneId The id of the scene.
 */
function CaravanConfinementController(sceneId) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = sceneId;
    SceneController.call(this, id);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.nextSceneReady = false;
    this.nextSceneName = "HamletAuction";
    this.nextFolderName = "Enslavement";

    this.misc = new MiscController(id, "misc");

    this.arrival = new ChoiceController(id, "arrival");

    this.name = new ChoiceController(id, "name");

    this.postName = new ChangeController(id, "postName", function () {
        var choice = that.name.get();
        if (choice === CaravanConfinementNameChoice.GROWL) {
            global.kobold.history.shared.enslavement.nightOne.complianceRating += 1;
            global.kobold.status.mental.decrease(3, "Threatened and insulted");
        } else if (choice === CaravanConfinementNameChoice.GLARE) {
            global.kobold.history.shared.enslavement.nightOne.complianceRating += 2;
            global.kobold.status.mental.decrease(2, "Getting yelled at");
        } else {
            global.kobold.history.shared.enslavement.nightOne.complianceRating += 3;
            global.kobold.status.mental.decrease(1, "Crude joke");
        }
    }, function () {
        var choice = that.name.get();
        if (choice === CaravanConfinementNameChoice.GROWL) {
            return global.ui.change.buildNotification([
                new Change("Called a feral", "Sanity", -1),
                new Change("Threatened slap", "Sanity", -2)
            ]);
        } else if (choice === CaravanConfinementNameChoice.GLARE) {
            return global.ui.change.buildNotification([
                new Change("Yelled at", "Sanity", -2)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Crude joke", "Sanity", -1)
            ]);
        }
    });

    this.mouth = new ChoiceController(id, "mouth");

    this.postMouth = new ChangeController(id, "postMouth", function () {
        var choice = that.mouth.get();
        if (choice === CaravanConfinementMouthChoice.BITE) {
            global.kobold.history.shared.enslavement.nightOne.complianceRating += 1;
            global.kobold.status.physical.decrease(8, "Minor abuse");
            global.kobold.status.mental.decrease(6, "Threatened and examined");
        } else if (choice === CaravanConfinementMouthChoice.TEETH) {
            global.kobold.history.shared.enslavement.nightOne.complianceRating += 2;
            global.kobold.status.mental.decrease(6, "Threatened and graded");
        } else {
            global.kobold.history.shared.enslavement.nightOne.complianceRating += 3;
            global.kobold.status.mental.decrease(3, "Examined and graded");
        }
    }, function () {
        var choice = that.name.get();
        if (choice === CaravanConfinementMouthChoice.BITE) {
            return global.ui.change.buildNotification([
                new Change("Slapped", "Health", -3),
                new Change("Choked", "Health", -5),
                new Change("Threatened", "Sanity", -5),
                new Change("Examined", "Sanity", -1)
            ]);
        } else if (choice === CaravanConfinementMouthChoice.TEETH) {
            return global.ui.change.buildNotification([
                new Change("Threatened", "Sanity", -3),
                new Change("Examined", "Sanity", -1),
                new Change("Graded", "Sanity", -2)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Examined", "Sanity", -1),
                new Change("Graded", "Sanity", -2)
            ]);
        }
    });

    this.tight = new ChoiceController(id, "tight");

    this.postTight = new ChangeController(id, "postTight", function () {
        var choice = that.tight.get();
        if (choice === CaravanConfinementTightnessChoice.CLAW) {
            global.kobold.history.shared.enslavement.nightOne.complianceRating += 1;
            global.kobold.status.mental.decrease(8, "Unwillingly touched");
            global.kobold.status.exertion.increase(5, "Loins probed");
        } else if (choice === CaravanConfinementTightnessChoice.STILL) {
            global.kobold.history.shared.enslavement.nightOne.complianceRating += 2;
            global.kobold.status.mental.decrease(8, "Reluctantly touched");
            global.kobold.status.exertion.increase(5, "Loins probed");
        } else {
            global.kobold.history.shared.enslavement.nightOne.complianceRating += 3;
            global.kobold.status.exertion.increase(5, "Loins probed");
        }
    }, function () {
        var choice = that.tight.get();
        if (choice === CaravanConfinementTightnessChoice.CLAW) {
            return global.ui.change.buildNotification([
                new Change("Unwillingly touched", "Sanity", -5),
                new Change("Meekness exposed", "Sanity", -3),
                new Change("Loins probed", "Sexual exertion", +5, true)
            ]);
        } else if (choice === CaravanConfinementTightnessChoice.STILL) {
            return global.ui.change.buildNotification([
                new Change("Unwillingly touched", "Sanity", -5),
                new Change("Loins probed", "Sexual exertion", +5, true)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Loins probed", "Sexual exertion", +5, true)
            ]);
        }
    });

    this.cart = new ChoiceController(id, "cart");

    this.bang = new ChangeController(id, "bang", function () {
        global.kobold.status.physical.decrease(23, "Bashed head against floor");
        global.kobold.status.mental.decrease(12, "Immense pain");
        global.kobold.stats.increaseStat(StatsEnum.ENDURANCE, 1);
        global.kobold.stats.decreaseStat(StatsEnum.INTELLIGENCE, 1);
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Hurt self", "Health", -18),
            new Change("Passed out", "Health", -5),
            new Change("Pain without gain", "Sanity", -12),
            new Change("Endured pain", "Endurance", +1),
            new Change("Did something stupid", "Intelligence", -1)
        ]);
    });

    this.count = new ChangeController(id, "count", function () {
        global.kobold.status.mental.increase(7, "Fell asleep");
        global.kobold.stats.increaseStat(StatsEnum.INTELLIGENCE, 1);
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Keeping busy", "Sanity", +2),
            new Change("Dozed off", "Sanity", +5),
            new Change("Practiced counting", "Intelligence", +1)
        ]);
    });

    this.gaze = new ChangeController(id, "gaze", function () {
        global.kobold.status.mental.increase(8, "Fell asleep");
        global.kobold.stats.increaseStat(StatsEnum.LUCK, 1);
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Tranquility", "Sanity", +3),
            new Change("Dozed off", "Sanity", +5),
            new Change("Star-blessed", "Luck", +1)
        ]);
    });

    this.talk = new ChangeController(id, "talk", function () {
        global.kobold.status.mental.increase(10, "Onesided conversation");
        global.kobold.stats.increaseStat(StatsEnum.CHARISMA, 1);
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Onesided conversation", "Sanity", +10),
            new Change("Practiced talking to people", "Charisma", +1)
        ]);
    });

    this.breakFail = new ChangeController(id, "breakFail", function () {
        global.kobold.status.stamina.decrease(5, "Struggling");
        global.kobold.stats.increaseStat(StatsEnum.STRENGTH, 1);
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Struggling", "Stamina", -5),
            new Change("Workout", "Strength", +1)
        ]);
    });

    this.breakSucceed = new ChangeController(id, "breakSucceed", function () {
        global.kobold.history.shared.enslavement.nightOne.complianceRating -= 2;
        global.kobold.status.physical.decrease(5, "Rope burn");
        global.kobold.status.stamina.decrease(5, "Struggling");
        global.kobold.status.mental.increase(10, "Newfound freedom");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Rope burn", "Health", -5),
            new Change("Struggling", "Stamina", -5),
            new Change("Hope", "Sanity", +5),
            new Change("Freedom", "Sanity", +5)
        ]);
    });

    this.breakSucceedContinue = new ChangeController(id, "breakSucceedContinue", function () {
        global.kobold.status.physical.decrease(20, "Head bashed");
        global.kobold.status.mental.decrease(15, "Hope shattered");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Head bashed", "Health", -15),
            new Change("Knocked out", "Health", -5),
            new Change("Hope shattered", "Sanity", -15)
        ]);
    });

    this.slipFail = new ChangeController(id, "breakFail", function () {
        global.kobold.status.stamina.decrease(5, "Struggling");
        global.kobold.stats.increaseStat(StatsEnum.AGILITY, 1);
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Struggling", "Stamina", -5),
            new Change("Workout", "Agility", +1)
        ]);
    });

    this.slipSucceed = new ChangeController(id, "breakSucceed", function () {
        global.kobold.history.shared.enslavement.nightOne.complianceRating -= 2;
        global.kobold.status.physical.decrease(5, "Rope burn");
        global.kobold.status.stamina.decrease(5, "Struggling");
        global.kobold.status.mental.increase(10, "Newfound freedom");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Rope burn", "Health", -5),
            new Change("Struggling", "Stamina", -5),
            new Change("Hope", "Sanity", +5),
            new Change("Freedom", "Sanity", +5)
        ]);
    });

    this.slipSucceedContinue = new ChangeController(id, "slipSucceedContinue", function () {
        global.kobold.status.physical.decrease(20, "Head bashed");
        global.kobold.status.mental.decrease(15, "Hope shattered");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Head bashed", "Health", -15),
            new Change("Knocked out", "Health", -5),
            new Change("Hope shattered", "Sanity", -15)
        ]);
    });

    this.preContinue = new ContinueController(id, "preContinue");

    //this.splashSound = new AudioController(id, "res/audio/splash.mp3", 50);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

}
