/**
 * Bang section of Enslavement, Night One.
 * @param {CaravanConfinementController} controller The Enslavement, Night One controller.
 */
function CaravanConfinementBang(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = CaravanConfinementCartChoice.BANG;
    var reqController = ctrl.cart;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You decide it'd be best to at least [b]try[/b] to escape the bandits' clutches as soon as possible.
                            In fact, why not start straight away?
                            The wood the floor of this cart is made out of looks kind of [i]weak[/i].
                            It's seen better days, and you doubt it was very sturdy to begin with.
                            You figure you might be able to break it.
                        [/p]
                        [p]
                            But with what?
                            Your arms are tied, and laying on the ground, your legs aren't of much use.
                            If you get up, a guard might see what you're doing.
                            That leaves only one option: your head.
                        [/p]
                        [p]
                            You sigh loudly.
                            You [b]need[/b] to get out of here, and this is the only way you're going to accomplish that.
                            You slowly lift your head off the floor, before [b]bashing[/b] it against the frail wood.
                            You barely refrain from yelping out in pain, tears forming at the corners of your eyes.
                            Your head feels like it cracked open as much as the wood did, if not more.
                        [/p]
                        [p]
                            While hurting, you need to press on.
                            You've already taken the first step.
                            You're already [b]in pain[/b].
                            It'd be an outright [b]waste[/b] to stop now.
                            Gritting your teeth, your rear your head back once more.
                            Again, you [b]slam[/b] it downwards, your already bruised forehead feeling like it's about to [b]explode[/b].
                            The wood creaks, but it does not crack.
                            Just [b]once more[/b].
                        [/p]
                        [p]
                            Your teeth chattering, you lift your head for a third time.
                            You close your eyes.
                            It hurts [b]so much[/b].
                            Your breathing is ragged.
                            You [b]need[/b] to do this.
                            Swallowing deeply, you [b]bash[/b] the floor with your forehead a third time.
                        [/p]
                        [p]
                            A loud [b]crack[/b] resonates throughout the cart.
                            You open your eyes.
                            Blood stains the floor, which seems to have held, despite your best efforts.
                            That must mean that thing that cracked must have been...your head?
                        [/p]
                        [p]
                            Suddenly, your sight goes blurry.
                            Soon after, it fades to black.
                            You slump down lifelessly onto the wooden wagon floor.
                            Your breathing slows to a crawl.
                            You pass out, the last thought on your mind being that
                            perhaps this wasn't the brightest idea after all.
                        [/p]`;

        return global.parser.parse(toReturn);
    }
    
    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.bang.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}
