/**
 * Gaze section of Enslavement, Night One.
 * @param {CaravanConfinementController} controller The Enslavement, Night One controller.
 */
function CaravanConfinementGaze(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = CaravanConfinementCartChoice.GAZE;
    var reqController = ctrl.cart;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            With nothing better to do, you decide to pass the time by gazing at the stars,
                            which you can see clearly through the metal bars of your [i]cell[/i].
                            You've always been fascinated with celestial things.
                            Those twinkling torches high up in the night sky have been the object of your dreams for many years now.
                        [/p]
                        [p]
                            You roll onto your back, and stare into the void.
                            The pitch blackness dotted with sparkling stars is soothing and easing.
                            You've been through [b]hell[/b] today.
                            You're glad to finally have a quiet moment in which you can [i]unwind[/i] and [b]think[/b] about things.
                        [/p]
                        [p]
                            Sure, you've been captured, but this isn't the first time you've botched a heist.
                            You've never actually been in a [i]cell[/i] before, though.
                            In the past, you've always managed to slip out of the greasy palms of whatever guardsman or vigilante caught you [i]before[/i] getting imprisoned.
                            Oh well, first time for everything.
                            You'll find a way to escape yet.
                            Just need to wait for the perfect opportunity to come along.
                        [/p]
                        [p]
                            You begin to relax, your eyelids growing more weary with every passing second.
                            Suddenly, it seems as though your cell is illuminated a little bit more than usual.
                            Like the stars themselves are shining their light down upon you.
                            After a few moments, the sudden brightness fades, leaving nothing but a lasting impression.
                            Come to think of it, your eyes weren't even open when it happened.
                            Perhaps you were already dreaming.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.gaze.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}
