/**
 * Hide section of the woods.
 * @param {ForestController} controller The Forest controller.
 */
function ForestWoodsHide(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstChoice = new ForestWoodsHideTree(ctrl);
    var secondChoice = new ForestWoodsHideBush(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You make the wise decision to hide from your pursuers.
                            You stole [b]a lot[/b] of money, and you don't think your thievery went entirely unnoticed.
                            If they catch you, who knows what those guards will do?
                            They could execute you on the spot if they so please,
                            or torture you until you [b]wish[/b] they [b]had[/b].
                            There are fates worse than death, which are, sadly enough, often reserved for [i]your kind[/i].
                            You don't want to find out first-hand what those fates entail.
                        [/p]
                        [p]
                            The forest is a big place, with plenty of hidden hidey-holes.
                            It is your home, and you know it inside and out.
                            Well, [i]most[/i] of it anyway.
                            This area in particular, you don't really have any hideaways in.
                            Still, there are plenty of apt places for you to hide.
                            Within a few minutes of exploration,
                            you're bound to find a hollow tree stump or some overgrown roots for you to conceal yourself in.
                        [/p]
                        [p]
                            Right as you were about to [i]start[/i] looking for a hiding spot, you become aware of a faint noise in the distance.
                            Is that...barking?
                            It's getting louder.
                            Definitely a feral dog or a wolf of some kind.
                            You don't think wolves actually bark, though, so it [b]must[/b] be a dog.
                            A tracking dog, perhaps?
                            Surely the caravan guards didn't bring one of those, right?
                            To hunt down sneaky kobold thieves like yourself with?
                            Another bark.
                            It's getting closer.
                            [i]Oh no.[/i]
                        [/p]
                        [p]
                            All you needed was five minutes.
                            Five minutes, and you'd have been able to conceal yourself [b]so well[/b] they would [b]never[/b] have found you.
                            Your mind races a million miles an hour, trying to come up with a [b]backup plan[/b], a place to hide in [b]right now[/b].
                            Alas, there's nothing around but [b]trees[/b] and [b]bushes[/b].
                            Wait a minute.
                            Trees.
                            And bushes.
                            You could hide in those!
                            Better hurry, you doubt you have long before the guards arrive.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = ctrl.hide.has() ? "disabled" : "";
        var isSelected = ctrl.hide.get() === ForestWoodsHideChoice.TREE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>A tree</button>`;

        var tooltip = global.parser.parse(`Try to climb a tree in order to stay hidden.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = ctrl.hide.has() ? "disabled" : "";
        var isSelected = ctrl.hide.get() === ForestWoodsHideChoice.BUSH;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>A bush</button>`;

        var tooltip = global.parser.parse(`Hide in a nearby bush.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You hide in: </div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.hide.set(ForestWoodsHideChoice.TREE);
            loadNextPart(firstChoice);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.hide.set(ForestWoodsHideChoice.BUSH);
            loadNextPart(secondChoice);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.woods.getArrivalChoice() !== ForestWoodsArrivalChoiceEnum.HIDE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += firstChoice.build();
        part += secondChoice.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.woods.getArrivalChoice() !== ForestWoodsArrivalChoiceEnum.HIDE) {
            return;
        }

        firstChoice.registerListeners();
        secondChoice.registerListeners();

        if (ctrl.hide.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}

var ForestWoodsHideChoice = {
    TREE: 1,
    BUSH: 2
};
