/**
 * Unknown section of the forest scene.
 * @param {ForestController} controller The forest controller.
 */
function ForestUnknown(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var pryId = "unknownPryButton";
    var yankId = "unknownYankButton";

    var toPry = new ForestUnknownPry(ctrl);
    var toYank = new ForestUnknownYank(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[st]Deepwoods exploration[/st]
                        [p]
                            Your quick breather over, you decide to journey into parts of the forest you've never been to before.
                            You tend to avoid these deeper stretches of woodland, as you've heard plenty of [i]stories[/i] of the [i]things[/i] that dwell there.
                            Roaming packs of feral predators, human-sized spiders, uncivilized tribes of various species,
                            and even a few green dragons all call the savage, hardly ever explored deepwoods their home.
                        [/p]
                        [p]
                            You're certain that whoever may be pursuing you, too, is aware of the dangers of the uncharted regions of the forest.
                            You hope they won't have the guts or the will to follow you on your reckless venture into the unknown.
                            And if they do, at least you won't be the [i]only[/i] morsel around for the various woodland beasts to hunt.
                        [/p]
                        [p]
                            Neither uncivilized tribes nor feral animals would have much interest in robbing you of your stolen hoard of gold,
                            save for perhaps fashioning the individual coins into trinkets. 
                            In that sense, you suppose the deepwoods are just a tiny hint [i]safer[/i] than the bandit-infested surroundings of Varanar, at least for you.
                            Still, the lack of highwaymen doesn't nearly make up for myriad of other dangers.
                        [/p]
                        [p]
                            The moment you step off the beaten path, you instantly notice the forest growing more [b]dense[/b].
                            The underbrush is thicker and more prevalent, and the trees are packed closely together.
                            The canopy blocks most of the sunlight, making this area far less illuminated than the rest of the woods, as though enveloped in a perpetual dusk.
                            A darkness which you fear will only deepen after the sun has set.
                        [/p]
                        [p]
                            About half an hour into your journey, you come across a large, thorny bush sitting in the middle of a small clearing.
                            At first, you think nothing of it, but as you walk past, you notice something familiar hanging off the side of the bush.
                            It's your old ${global.kobold.inventory.createItemWithTooltip(new WoolSweater())}, which you lost about half a week ago!
                        [/p]
                        [p]
                            How did that end up [i]here[/i]?
                            Admittedly, when you lost it, you weren't paying all too much attention.
                            You were too busy running away from a bunch of angry humans to mind your surroundings.
                            But still, you're fairly certain you didn't lose it [i]here[/i].
                            In fact, you don't think you've [b]ever[/b] been to this part of the woods before,
                            not even [i]while[/i] absent-mindedly fleeing farmers with pitchforks.
                        [/p]
                        [p]
                            To make matters even more confusing, you lost your shirt to a thornbush that looks [b]just[/b] like this one.
                            Did someone find the garment, bring it here and then lose it again to a very similar plant?
                            Unlikely.
                            Did someone move the bush?
                            Why would they have gone through the trouble of doing that, without so much as removing your shirt from its thorny exterior?
                            Did the plant itself [i]move[/i]?
                            Nonsense.
                            Plants can't move.
                        [/p]
                        [p]
                            You're probably just misremembering exactly where you lost your sweater.
                            Come to think of it, that entire day is a bit [i]hazy[/i] in your memory.
                            Besides, the important thing isn't how or where you lost the shirt.
                            You found it once again, and that's what matters.
                            Now all you need to do is retrieve it without getting yourself stung.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the ant choice button.
     */
    function buildPryChoice() {
        var disabled = ctrl.unknown.base.has() ? "disabled" : "";
        var isSelected = ctrl.unknown.base.get() === ForestUnknownChoice.PRY;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${pryId}" ${disabled}>Pry it off</button>`;

        var tooltip = `Carefully pry off your sweater, to prevent it from getting damaged.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 165px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the shake choice button.
     */
    function buildYankChoice() {
        var disabled = ctrl.unknown.base.has() ? "disabled" : "";
        var isSelected = ctrl.unknown.base.get() === ForestUnknownChoice.YANK;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${yankId}" ${disabled}>Yank it off</button>`;

        var tooltip = `Crudely yank your shirt off of the bush.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildPryChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildYankChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + pryId), $("#" + yankId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerPryListener() {
        $("#" + pryId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unknown.base.set(ForestUnknownChoice.PRY);
            loadNextPart(toPry);
        });

    }

    /**
     * Adds a listener to the button.
     */
    function registerYankListener() {
        $("#" + yankId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unknown.base.set(ForestUnknownChoice.YANK);
            loadNextPart(toYank);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "forestUnknown";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.woods.getArrivalChoice() !== ForestWoodsArrivalChoiceEnum.UNKNOWN) {
            return "";
        }
        if(!ctrl.unknown.base.has()) {
            ctrl.unknown.introSound.playSettingsDependent();
        }
        global.ui.applyTheme("deepwoods");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += toPry.build();
        part += toYank.build();
        part += `</div>`;

        // BUILD CHILDREN HERE

        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.woods.getArrivalChoice() !== ForestWoodsArrivalChoiceEnum.UNKNOWN) {
            return;
        }

        toPry.registerListeners();
        toYank.registerListeners();

        if (ctrl.unknown.base.has()) {
            return;
        }

        registerPryListener();
        registerYankListener();
    };
}

var ForestUnknownChoice = {
    PRY: 1,
    YANK: 2
};
