/**
 * Barn section of the farm scene.
 * @param {BreedingFarmController} controller The breeding farm controller.
 */
function BarnAttack(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var lootId = "unawareAttackLootButton";
    var freeId = "unawareAttackFreeButton";

    var toLoot = new BarnAttackLoot(ctrl);
    var toFree = new BarnAttackFree(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon();
        var toReturn = `[p]
                            You wield your ${global.kobold.inventory.createItemWithTooltip(weapon)} and slowly, stealthily approach the vulpine from behind.
                            Being a fox, she may have incredible hearing and a sense of smell well beyond your own,
                            but you are a [b]kobold[/b], one of the sneakiest races around.
                            That, and she's far too occupied with searching through an entire bag full of noisy rubbish to notice you drawing close.
                        [/p]`;
        
        if(weapon.stab()) {
            global.kobold.history.shared.cassidy.dead = true;
            toReturn += `[p]
                            "[color=orange]Aha![/color]" the vixen exclaims, rising to her feet with a single filled vial.
                            Just as you were about to strike, she turns around, audibly gasping at the sight of her captive brandishing a weapon.
                            Adrenaline courses through your veins.
                            Your ambush foiled, your fight or flight instinct kicks in.
                            With a loud, menacing growl of sheer rage, you leap towards Cassidy, weapon in hand.
                        [/p]
                        [p]
                            Your body collides with the fox's slender frame, knocking the stunned vixen off her feet.
                            You fall ontop of the backwards-tumbling woman as she lands on the ground with a heavy thud.
                            You come face to face with the [b]bitch[/b] that tried to [b]ruin[/b] your life.
                            You stare deeply into her shocked, wide-open eyes.
                            Her jaw falls agape.
                            A single [i]gurgle[/i] escapes her lips.
                            Her eyes slowly close, and her body slumps down lifelessly.
                        [/p]
                        [p]
                            Looking downwards, you find your blade having sunk into Cassidy's very heart, felling her with a single lethal [b]stab[/b].
                            You must have managed to get off a lucky strike during your initial lunge, or perhaps when the both of you fell to the ground.
                            Regardless, the deed is done.
                            You pull your weapon from the canine's corpse, and wipe the blood it is stained with off onto the ground.
                            You take a few deep breaths, excitement rushing from your system.
                            [b]You've just killed Cassidy.[/b]
                        [/p]
                        [p]
                            Calming down a little, you rise to your feet.
                            You convince yourself that you did what [b]had[/b] to be done, no matter how horrible of an experience it was.
                            You had no choice.
                            It was her, or you.
                            Besides, she was a [b]terrible[/b] person.
                            She [b]deserved[/b] this.
                            Or so you tell yourself, to dull your aching conscience.
                        [/p]`;
        } else {
            toReturn += `[p]
                            You brace your weapon, holding it high above your head, preparing your entire body to strike down with all of your might.
                            You fail to contain a loud, angry growl, alerting the vixen to your presence directly behind her, yet before she even manages to react,
                            you bring your ${global.kobold.inventory.createItemWithTooltip(weapon)} down upon her head,
                            the hard surface impacting against the woman's skull with a twistingly satisfying [b]crack[/b].
                        [/p]
                        [p]
                            The canine's body slumps down to the ground, completely lifeless.
                            For a moment, you fear you might have [b]killed[/b] her, but then you notice the vixen's bare, 
                            prone torso slowly, yet rhythmically inflating and deflating.
                            Still breathing.
                            You didn't kill her, you've just managed to knock her unconscious in a single blow.
                            Righteous retribution.
                        [/p]
                        [p]
                            Satisfied with your work, you take a few deep breaths to calm yourself down.
                            You did it.
                            You [b]won[/b].
                            That [b]bitch[/b] won't be turning you into a breeding kobold, or worse yet some rich guy's side dish.
                            Not today.
                            Not [b]ever[/b].
                        [/p]`;
        }
        
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildLootChoice() {
        var disabled = ctrl.unawareAttack.has() ? "disabled" : "";
        var isSelected = ctrl.unawareAttack.get() === BarnAttackChoice.LOOT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${lootId}" ${disabled}>Loot</button>`;
        var tooltip = `Steal whatever you can, then run away from the scene of the crime.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFreeChoice() {
        var disabled = ctrl.unawareAttack.has() ? "disabled" : "";
        var isSelected = ctrl.unawareAttack.get() === BarnAttackChoice.FREE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${freeId}" ${disabled} style="width:160px;">Free your peers</button>`;
        var tooltip = `Open the cubicles and free the other kobolds. Surely some of them must still be able-minded, right?`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        button = `<div class="inlinechoice" style="width:160px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildLootChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildFreeChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + lootId), $("#" + freeId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }
    
    /**
     * Adds a listener to the button.
     */
    function registerLootListener() {
        $("#" + lootId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unawareAttack.set(BarnAttackChoice.LOOT);
            loadNextPart(toLoot);
        });
    }
    
    /**
     * Adds a listener to the button.
     */
    function registerFreeListener() {
        $("#" + freeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unawareAttack.set(BarnAttackChoice.FREE);
            loadNextPart(toFree);
        });
    }
    

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "barnAttack";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if(ctrl.unaware.get() !== BarnUnawareChoice.ATTACK) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.unawareAttack.change(ctrl.misc.weapon());
        part += buildChoice();
        part += toLoot.build();
        part += toFree.build();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if(ctrl.unaware.get() !== BarnUnawareChoice.ATTACK) {
            return;
        }
        
        toLoot.registerListeners();
        toFree.registerListeners();

        if (ctrl.unawareAttack.has()) {
            return;
        }

        registerLootListener();
        registerFreeListener();
    };
}

var BarnAttackChoice = {
    LOOT: 1,
    FREE: 2
};
