/**
 * Unsaved section of the home scene.
 * @param {HomeController} controller The home controller.
 */
function HomeUnsaved(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var insideId = "unsavedInsideButton";
    var hammockId = "unsavedHammockButton";
    var groundId = "unsavedGroundButton";

    var toInside = new HomePillaged(ctrl);
    var toHammock = new HomeHammock(ctrl);
    var toGround = new HomeGround(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[st]Late arrival[/st]`;
        toReturn += `   [p]
                            After a short trek through the forest, you begin to near your small hovel, located near the northern edge of the woods.
                            Thank goodness you're not far off, because night is upon you, 
                            and you know how [i]dangerous[/i] things can get around the woodland after the sun goes down.
                        [/p]
                        [p]
                            Your wooden [i]shed[/i], illuminated dimly by moonlight shining through the treetops, soon comes into view.
                            As you approach, however, you notice something's [i]wrong[/i].
                            The door to your humble abode hangs crooked on its sturdy hinges, obviously [b]forced[/b] open.
                            You've no doubt that this was the work of the Three-Tailed Rat's men.
                            The only question that lingers in your mind is whether or not they're still in the area.
                        [/p]
                        [p]
                            You skulk around your shoddy shack, keeping your eyes peeled and your ears open,
                            yet you can't quite find anyone at first glance.
                            You reckon they must have left after discovering that your home contains nothing but the hay you sleep on.
                            Too bad for them, you carry about all you own, which isn't really all that much.
                        [/p]
                        [p]
                            You let out a loud yawn, your mind weary from the [i]active[/i] day you've had, worn out both physically as well as emotionally.
                            While you'd love to pull an all-nighter just to see if any of those no-good [b]looters[/b] would have the [b]guts[/b] to return 
                            to your shack while you're present, both your body and your brain are [b]far[/b] too tired to stay up for much longer, let alone the entire night.
                        [/p]
                        [p]
                            However sleepy you may be, you're still alert enough to realize that [i]should[/i] whoever raided your home return to find you sleeping,
                            that would [b]not[/b] end well for you.
                            Then again, why [i]would[/i] they return?
                            They didn't find anything of value, and you doubt anyone would come looking for you in the middle of night.
                            But with the Three-Tailed Rat Clan, you never know.
                        [/p]
                        [p]
                            Besides sleeping in what remains of your shed, there's also the possibility of resting outdoors,
                            which might prove to be a little more safe.
                        [/p]
                        [p]
                            A few months ago, you found a fisherman's abandoned net washed up on the bank of the river.
                            Instead of letting it go to waste, you tied up the intertwined rope-webbing in between two trees,
                            fashioning yourself a hammock not too far off from your home.
                            While undoubtedly more [i]comfortable[/i] than sleeping on the ground,
                            you would also be quite a bit more [i]visible[/i] and [i]vulnerable[/i] as opposed to sleeping in a bush.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the inside choice button.
     */
    function buildInsideChoice() {
        var disabled = ctrl.unsaved.hasMadeSleepChoice() ? "disabled" : "";
        var isSelected = ctrl.unsaved.getSleepChoice() === HomeUnsavedChoice.INSIDE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${insideId}" ${disabled}>Inside</button>`;

        var tooltip = `Your home may be plundered, but you doubt they took <span class="italic">all</span> of your hay.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the hammock choice button.
     */
    function buildHammockChoice() {
        var disabled = ctrl.unsaved.hasMadeSleepChoice() ? "disabled" : "";
        var isSelected = ctrl.unsaved.getSleepChoice() === HomeUnsavedChoice.HAMMOCK;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${hammockId}" ${disabled} style="font-size: 18px;">On the hammock</button>`;

        var tooltip = `The hammock is comfortable enough to get some decent rest in, but you'd be pretty exposed during the night.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the ground choice button.
     */
    function buildGroundChoice() {
        var disabled = ctrl.unsaved.hasMadeSleepChoice() ? "disabled" : "";
        var isSelected = ctrl.unsaved.getSleepChoice() === HomeUnsavedChoice.GROUND;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${groundId}" ${disabled}>On the ground</button>`;

        var tooltip = `Your home is no longer safe, and you'd rather not sleep in a strung-up net with a bunch of people potentially hunting you.
                       Rest on the cold ground, in a nearby bush.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 300px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You sleep: </div>`;
        choice += buildInsideChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildHammockChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildGroundChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + insideId), $("#" + hammockId), $("#" + groundId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerInsideListener() {
        $("#" + insideId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unsaved.makeSleepChoice(HomeUnsavedChoice.INSIDE);
            loadNextPart(toInside);
        });

    }

    /**
     * Adds a listener to the button.
     */
    function registerHammockListener() {
        $("#" + hammockId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unsaved.makeSleepChoice(HomeUnsavedChoice.HAMMOCK);
            loadNextPart(toHammock);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerGroundListener() {
        $("#" + groundId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unsaved.makeSleepChoice(HomeUnsavedChoice.GROUND);
            loadNextPart(toGround);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "homeUnsaved";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.status() !== HomeStatusEnum.UNSAVED) {
            return "";
        }
        if (!ctrl.unsaved.hasMadeSleepChoice()) {
            ctrl.introAudio.playSettingsDependent();
        }
        global.ui.applyTheme("home_night");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += toInside.build();
        part += toHammock.build();
        part += toGround.build();
        part += `</div>`;

        // BUILD CHILDREN HERE

        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.status() !== HomeStatusEnum.UNSAVED) {
            return;
        }

        toInside.registerListeners();
        toHammock.registerListeners();
        toGround.registerListeners();

        if (ctrl.unsaved.hasMadeSleepChoice()) {
            return;
        }

        registerInsideListener();
        registerHammockListener();
        registerGroundListener();
    };
}

var HomeUnsavedChoice = {
    INSIDE: 1,
    HAMMOCK: 2,
    GROUND: 3
};
