/**
 * The first part of the introduction.
 * @param {IntroductionController} controller The controller.
 * @param {IntroductionSecondPart} secondPart The next part to load once finished.
 */
function IntroductionFirstPart(controller, secondPart) {
    // PRIVATE MEMBERS \\       var private = "test";
    var ctrl = controller;
    var nextPart = secondPart;
    var start =  `  <div class="title fancy">
                         introduction
                    </div>
                    <div class="text">
                        You are a <span class="bold">kobold</span>. You are small, flexible, nimble and rather stealthy when you want
                        to be. On the other hand you are naive, not very strong, kind of a pushover and 
                        generally not very good in a fight. You make up for your lack of combat skills
                        by a driven determination not to fall prey to the many dangers of this world.
                    </div>

                    <div class="text">
                        The area you live in is mostly predominated by humans and anthros, who see you as a lesser species. 
                        Kobolds officially have no rights, so the sight of one of your kind getting raped, 
                        beaten, sold as a slave or even killed in broad daylight is, unfortunately for you, not so uncommon.
                    </div>

                    <div class="text">
                        Luckily, you have managed to avoid the worst so far, by making your home in the outstretched 
                        woods surrounding the human capital of <span class="bold">Varanar</span>. While the city itself is 
                        a melting pot for all walks of life, the forest isn't nearly as diverse.
                    </div>
    
                    <div class="text">
                        Still, there's plenty to be afraid of. 
                        Wolves, bobcats, forest wyverns, gryphons, you name it.
                        Heck, even something as docile as a deer, when provoked, could pose a threat to a small kobold like you.
                        To make matters worse, there are rumors going around of a hulking mass of tentacles skulking through the darkness, 
                        violating who or <span class="italic">what</span>ever it gets in its grip.
                    </div>

                    <div class="text">
                        On top of the feral dangers, there are various bandits and outlaws 
                        which have made these woods their home. Many of them band together, forming
                        fickle alliances that often end in backstabbing and betrayal, all in pursuit
                        of coin. They prey on unsuspecting victims, to rob them of their 
                        riches, their decency, their life or all three.
                    </div>
    
                    <div class="text">
                        Some of these groups are comprised exclusively of kobolds.
                        One of them is the <span class="bold">Three-Tailed Rat Clan</span>.
                        Based in Varanar, this organized gang of ruthless, thick-scaled thugs is lead by 
                        the only non-kobold member, a man known as <span class="bold">The Rat</span>.
                        Their main source of income is extortion, coercing protection money from people
                        living in the more lawless regions of the land.
                        <span class="bold">Regions like the forest. And people like you.</span>
                    </div>`;
    
    var genderChoiceHeader =       `<div class="choicecontainer">
                                        <div class="inlinechoiceheader">
                                            You are:
                                        </div>`;
    
    var genderChoiceButtonMid =        `<hr class="inlinechoiceseparator">`;
    
    var genderChoiceFooter =    `</div>`;
    
    var genderChoiceButtonMaleId = "male";
    var genderChoiceButtonFemaleId = "female";
    
    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    
     /**
     * Builds and returns the gender choice.
     * The starting choice may be disabled if it has already been made.
     */
    function buildGenderChoice() {
        var choice = genderChoiceHeader;
        choice += buildGenderChoiceMaleButton();
        choice += genderChoiceButtonMid;
        choice += buildGenderChoiceFemaleButton();
        choice += genderChoiceFooter;
        return choice;
    }
    
    /**
     * Returns the male choice button.
     */
    function buildGenderChoiceMaleButton() {
        var disabled = ctrl.hasMadeGenderChoice() ? "disabled" : "";
        var selected = ctrl.hasMadeGenderChoice() && ctrl.getGenderChoice() == GenderEnum.MALE ? "selected" : "";
        return `<button class="button inlinechoice ${selected}" id="${genderChoiceButtonMaleId}" ${disabled}>
                    Male
                </button>`;
    }
    
    /**
     * Returns the female choice button.
     */
    function buildGenderChoiceFemaleButton() {
        var disabled = ctrl.hasMadeGenderChoice() ? "disabled" : "";
        var selected = ctrl.hasMadeGenderChoice() && ctrl.getGenderChoice() == GenderEnum.FEMALE ? "selected" : "";
        return `<button class="button inlinechoice ${selected}" id="${genderChoiceButtonFemaleId}" ${disabled}>
                    Female
                </button>`;
    }
        
    /**
     * Registers a click listener for the male gender choice button.
     */
    function registerMaleGenderChoiceListener() {
        $("#" + genderChoiceButtonMaleId).click(function(){
            $(this).addClass("selected");
            genderChoiceClick(GenderEnum.MALE);
        });
    }
    
    /**
     * Registers a click listener for the female gender choice button.
     */
    function registerFemaleGenderChoiceListener() {
        $("#" + genderChoiceButtonFemaleId).click(function(){
            $(this).addClass("selected");
            genderChoiceClick(GenderEnum.FEMALE);
        });
    }
    
    /**
     * Executed when a gender choice button is clicked.
     * @param {GenderEnum} choice The selected gender.
     */
    function genderChoiceClick(choice) {
        disableButtons();
        ctrl.makeGenderChoice(choice);
        var secondPart = nextPart.build(true);
        global.ui.appendBeforeFinisher(secondPart, global.ui.genericHeaderId);
        $("#" + nextPart.id).fadeIn(400);
        nextPart.registerListeners();
    }
    
    /**
     * Disables both gender choice buttons.
     */
    function disableButtons() {
        var maleButton = $("#" + genderChoiceButtonMaleId);
        var femaleButton = $("#" + genderChoiceButtonFemaleId);
        maleButton.unbind();
        femaleButton.unbind();
        maleButton.prop('disabled', true);
        femaleButton.prop('disabled', true);
    }
    
    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    
    /**
     * Builds and returns this part.
     */
    this.build = function() {
        return start + buildGenderChoice();
    };
    
    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function() {
        if(ctrl.hasMadeGenderChoice()){
            return;
        }
        registerMaleGenderChoiceListener();
        registerFemaleGenderChoiceListener();
    };
}
